#include <math.h>
#include "MPU9250.h"
uint8_t data[2];
double accelRange,gyroRange;
#define maximum_transfer 6
static uint8_t dummy_buffer[maximum_transfer];
char accelBuf[maximum_transfer+1];
volatile uint8_t tx_finished,rx_finished;
#define READ_FLAG   0x80

void MPU9250_beginAccel(uint8_t mode) {
  switch(mode) {
  case ACC_FULL_SCALE_2_G:
    accelRange = 2.0;
    break;
  case ACC_FULL_SCALE_4_G:
    accelRange = 4.0;
    break;
  case ACC_FULL_SCALE_8_G:
    accelRange = 8.0;
    break;
  case ACC_FULL_SCALE_16_G:
    accelRange = 16.0;
    break;
  default:
    return; // Return without writing invalid mode
  }
 	
	data[0]=USER_CTRL; data[1]=(1<<4);
	spi_transfer_dma((uint32_t)data,2);
	while(tx_finished==0){;}
		tx_finished=0;
		
	data[0]=MPU9250_ADDR_ACCELCONFIG; data[1]=mode;
	spi_transfer_dma((uint32_t)data,2);
	while(tx_finished==0){;}
		tx_finished=0;
}

uint8_t MPU9250_accelUpdate(void) {
	dummy_buffer[0]=0x3B|READ_FLAG;
	
	spi_receive_dma(accelBuf,7);
	spi_transfer_dma(dummy_buffer,7);
	while(rx_finished==0){;}
	rx_finished=0;
	return 0;
}


float MPU9250_accelGet(uint8_t highIndex, uint8_t lowIndex) {
  int16_t v = ((int16_t) accelBuf[highIndex]) << 8 | accelBuf[lowIndex];
  return ((float) -v) * accelRange / (float) 0x8000; // (float) 0x8000 == 32768.0
}



float MPU9250_accelX(void) {
  return MPU9250_accelGet(1, 2);
}

float MPU9250_accelY(void) {
  return MPU9250_accelGet(3, 4);
}

float MPU9250_accelZ(void) {
  return MPU9250_accelGet(5, 6);
}

	
void DMA2_Stream3_IRQHandler(void)
		{
			
		if(DMA2->LISR&(DMA_LISR_TCIF3))
				{
					tx_finished=1;
				
					DMA2_Stream3->CR&=~DMA_SxCR_EN;
					
					DMA2->LIFCR |=DMA_LIFCR_CTCIF3;
				}
				
		if(DMA2->LISR&(DMA_LISR_HTIF3))
				{
					

					DMA2->LIFCR |=DMA_LIFCR_CHTIF3;
				}
				
		
		if(DMA2->LISR&(DMA_LISR_TEIF3))
						{

						DMA2->LIFCR|=(DMA_LIFCR_CTEIF3);
						}
						
		if(DMA2->LISR&(DMA_LISR_DMEIF3))
						{

						DMA2->LIFCR|=(DMA_LIFCR_CDMEIF3);
						}
						
		if(DMA2->LISR&(DMA_LISR_FEIF3))
						{

						DMA2->LIFCR|=(DMA_LIFCR_CFEIF0);
						}
			NVIC_ClearPendingIRQ(DMA2_Stream3_IRQn);
			
		}
		
		
void DMA2_Stream2_IRQHandler(void)
		{
			
		if(DMA2->LISR&(DMA_LISR_TCIF2))
				{
					rx_finished=1;

					DMA2_Stream2->CR&=~DMA_SxCR_EN;

					DMA2->LIFCR |=DMA_LIFCR_CTCIF2;
				}
				
		if(DMA2->LISR&(DMA_LISR_HTIF2))
				{
					

					DMA2->LIFCR |=DMA_LIFCR_CHTIF2;
				}
				
		
		if(DMA2->LISR&(DMA_LISR_TEIF2))
						{

						DMA2->LIFCR|=(DMA_LIFCR_CTEIF2);
						}
						
		if(DMA2->LISR&(DMA_LISR_DMEIF2))
						{

						DMA2->LIFCR|=(DMA_LIFCR_CDMEIF2);
						}
						
		if(DMA2->LISR&(DMA_LISR_FEIF2))
						{

						DMA2->LIFCR|=(DMA_LIFCR_CFEIF2);
						}

			NVIC_ClearPendingIRQ(DMA2_Stream2_IRQn);
		}
